@@ -33,12 +33,11 @@ import (
 	"k8s.io/cli-runtime/pkg/resource"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
-	helmv2 "github.com/fluxcd/helm-controller/api/v2beta2"
+	helmv2 "github.com/fluxcd/helm-controller/api/v2"
 	kustomizev1 "github.com/fluxcd/kustomize-controller/api/v1"
 	fluxmeta "github.com/fluxcd/pkg/apis/meta"
 	"github.com/fluxcd/pkg/oci"
 	sourcev1 "github.com/fluxcd/source-controller/api/v1"
-	sourcev1b2 "github.com/fluxcd/source-controller/api/v1beta2"
 
 	"github.com/fluxcd/flux2/v2/internal/utils"
 )
@@ -64,7 +63,7 @@ You can also trace multiple objects with different resource kinds using <resourc
   
   # API Version and Kind can also be specified explicitly
   # Note that either both, kind and api-version, or neither have to be specified.
-  flux trace redis --kind=helmrelease --api-version=helm.toolkit.fluxcd.io/v2beta2 -n redis`,
+  flux trace redis --kind=helmrelease --api-version=helm.toolkit.fluxcd.io/v2 -n redis`,
 	RunE: traceCmdRun,
 }
 
@@ -223,7 +222,7 @@ func traceKustomization(ctx context.Context, kubeClient client.Client, ksName ty
 	ksReady := meta.FindStatusCondition(ks.Status.Conditions, fluxmeta.ReadyCondition)
 
 	var gitRepository *sourcev1.GitRepository
-	var ociRepository *sourcev1b2.OCIRepository
+	var ociRepository *sourcev1.OCIRepository
 	var ksRepositoryReady *metav1.Condition
 	switch ks.Spec.SourceRef.Kind {
 	case sourcev1.GitRepositoryKind:
@@ -240,8 +239,8 @@ func traceKustomization(ctx context.Context, kubeClient client.Client, ksName ty
 			return "", fmt.Errorf("failed to find GitRepository: %w", err)
 		}
 		ksRepositoryReady = meta.FindStatusCondition(gitRepository.Status.Conditions, fluxmeta.ReadyCondition)
-	case sourcev1b2.OCIRepositoryKind:
-		ociRepository = &sourcev1b2.OCIRepository{}
+	case sourcev1.OCIRepositoryKind:
+		ociRepository = &sourcev1.OCIRepository{}
 		sourceNamespace := ks.Namespace
 		if ks.Spec.SourceRef.Namespace != "" {
 			sourceNamespace = ks.Spec.SourceRef.Namespace
@@ -348,7 +347,7 @@ Status:          Unknown
 		Kustomization      *kustomizev1.Kustomization
 		KustomizationReady *metav1.Condition
 		GitRepository      *sourcev1.GitRepository
-		OCIRepository      *sourcev1b2.OCIRepository
+		OCIRepository      *sourcev1.OCIRepository
 		RepositoryReady    *metav1.Condition
 		Annotations        map[string]string
 	}{
@@ -388,10 +387,10 @@ func traceHelm(ctx context.Context, kubeClient client.Client, hrName types.Names
 	}
 	hrReady := meta.FindStatusCondition(hr.Status.Conditions, fluxmeta.ReadyCondition)
 
-	var hrChart *sourcev1b2.HelmChart
+	var hrChart *sourcev1.HelmChart
 	var hrChartReady *metav1.Condition
 	if chart := hr.Status.HelmChart; chart != "" {
-		hrChart = &sourcev1b2.HelmChart{}
+		hrChart = &sourcev1.HelmChart{}
 		err = kubeClient.Get(ctx, utils.ParseNamespacedName(chart), hrChart)
 		if err != nil {
 			return "", fmt.Errorf("failed to find HelmChart: %w", err)
@@ -401,38 +400,65 @@ func traceHelm(ctx context.Context, kubeClient client.Client, hrName types.Names
 
 	var hrGitRepository *sourcev1.GitRepository
 	var hrGitRepositoryReady *metav1.Condition
-	if hr.Spec.Chart.Spec.SourceRef.Kind == sourcev1.GitRepositoryKind {
-		hrGitRepository = &sourcev1.GitRepository{}
-		sourceNamespace := hr.Namespace
-		if hr.Spec.Chart.Spec.SourceRef.Namespace != "" {
-			sourceNamespace = hr.Spec.Chart.Spec.SourceRef.Namespace
-		}
-		err = kubeClient.Get(ctx, types.NamespacedName{
-			Namespace: sourceNamespace,
-			Name:      hr.Spec.Chart.Spec.SourceRef.Name,
-		}, hrGitRepository)
-		if err != nil {
-			return "", fmt.Errorf("failed to find GitRepository: %w", err)
-		}
-		hrGitRepositoryReady = meta.FindStatusCondition(hrGitRepository.Status.Conditions, fluxmeta.ReadyCondition)
-	}
-
-	var hrHelmRepository *sourcev1b2.HelmRepository
+	var hrHelmRepository *sourcev1.HelmRepository
 	var hrHelmRepositoryReady *metav1.Condition
-	if hr.Spec.Chart.Spec.SourceRef.Kind == sourcev1b2.HelmRepositoryKind {
-		hrHelmRepository = &sourcev1b2.HelmRepository{}
-		sourceNamespace := hr.Namespace
-		if hr.Spec.Chart.Spec.SourceRef.Namespace != "" {
-			sourceNamespace = hr.Spec.Chart.Spec.SourceRef.Namespace
+	var hrOCIRepository *sourcev1.OCIRepository
+	var hrOCIRepositoryReady *metav1.Condition
+	if hr.Spec.Chart == nil {
+		if hr.Spec.ChartRef != nil {
+			switch hr.Spec.ChartRef.Kind {
+			case sourcev1.OCIRepositoryKind:
+				hrOCIRepository = &sourcev1.OCIRepository{}
+				sourceNamespace := hr.Namespace
+				if hr.Spec.ChartRef.Namespace != "" {
+					sourceNamespace = hr.Spec.ChartRef.Namespace
+				}
+				err = kubeClient.Get(ctx, types.NamespacedName{
+					Namespace: sourceNamespace,
+					Name:      hr.Spec.ChartRef.Name,
+				}, hrOCIRepository)
+				if err != nil {
+					return "", fmt.Errorf("failed to find OCIRepository: %w", err)
+				}
+				hrOCIRepositoryReady = meta.FindStatusCondition(hrOCIRepository.Status.Conditions, fluxmeta.ReadyCondition)
+			}
+			kubeClient.Get(ctx, types.NamespacedName{
+				Namespace: hr.Spec.ChartRef.Namespace,
+				Name:      hr.Spec.ChartRef.Name,
+			}, hrOCIRepository)
 		}
-		err = kubeClient.Get(ctx, types.NamespacedName{
-			Namespace: sourceNamespace,
-			Name:      hr.Spec.Chart.Spec.SourceRef.Name,
-		}, hrHelmRepository)
-		if err != nil {
-			return "", fmt.Errorf("failed to find HelmRepository: %w", err)
+	} else {
+		if hr.Spec.Chart.Spec.SourceRef.Kind == sourcev1.GitRepositoryKind {
+			hrGitRepository = &sourcev1.GitRepository{}
+			sourceNamespace := hr.Namespace
+			if hr.Spec.Chart.Spec.SourceRef.Namespace != "" {
+				sourceNamespace = hr.Spec.Chart.Spec.SourceRef.Namespace
+			}
+			err = kubeClient.Get(ctx, types.NamespacedName{
+				Namespace: sourceNamespace,
+				Name:      hr.Spec.Chart.Spec.SourceRef.Name,
+			}, hrGitRepository)
+			if err != nil {
+				return "", fmt.Errorf("failed to find GitRepository: %w", err)
+			}
+			hrGitRepositoryReady = meta.FindStatusCondition(hrGitRepository.Status.Conditions, fluxmeta.ReadyCondition)
+		}
+
+		if hr.Spec.Chart.Spec.SourceRef.Kind == sourcev1.HelmRepositoryKind {
+			hrHelmRepository = &sourcev1.HelmRepository{}
+			sourceNamespace := hr.Namespace
+			if hr.Spec.Chart.Spec.SourceRef.Namespace != "" {
+				sourceNamespace = hr.Spec.Chart.Spec.SourceRef.Namespace
+			}
+			err = kubeClient.Get(ctx, types.NamespacedName{
+				Namespace: sourceNamespace,
+				Name:      hr.Spec.Chart.Spec.SourceRef.Name,
+			}, hrHelmRepository)
+			if err != nil {
+				return "", fmt.Errorf("failed to find HelmRepository: %w", err)
+			}
+			hrHelmRepositoryReady = meta.FindStatusCondition(hrHelmRepository.Status.Conditions, fluxmeta.ReadyCondition)
 		}
-		hrHelmRepositoryReady = meta.FindStatusCondition(hrHelmRepository.Status.Conditions, fluxmeta.ReadyCondition)
 	}
 
 	var traceTmpl = `
@@ -448,7 +474,7 @@ Namespace:      {{.HelmRelease.Namespace}}
 {{- if .HelmRelease.Spec.TargetNamespace }}
 Target:         {{.HelmRelease.Spec.TargetNamespace}}
 {{- end }}
-Revision:       {{.HelmRelease.Status.LastAppliedRevision}}
+Revision:       {{.HelmRelease.Status.LastAttemptedRevision}}
 {{- if .HelmReleaseReady }}
 Status:         Last reconciled at {{.HelmReleaseReady.LastTransitionTime}}
 Message:        {{.HelmReleaseReady.Message}}
@@ -515,6 +541,34 @@ Message:       {{.GitRepositoryReady.Message}}
 Status:        Unknown
 {{- end }}
 {{- end }}
+{{- if .OCIRepository }}
+---
+OCIRepository:   {{.OCIRepository.Name}}
+Namespace:       {{.OCIRepository.Namespace}}
+URL:             {{.OCIRepository.Spec.URL}}
+{{- if .OCIRepository.Spec.Reference }}
+{{- if .OCIRepository.Spec.Reference.Tag }}
+Tag:             {{.OCIRepository.Spec.Reference.Tag}}
+{{- else if .OCIRepository.Spec.Reference.SemVer }}
+Tag:             {{.OCIRepository.Spec.Reference.SemVer}}
+{{- else if .OCIRepository.Spec.Reference.Digest }}
+Digest:          {{.OCIRepository.Spec.Reference.Digest}}
+{{- end }}
+{{- end }}
+{{- if .OCIRepository.Status.Artifact }}
+Revision:        {{.OCIRepository.Status.Artifact.Revision}}
+{{- end }}
+{{- if .OCIRepositoryReady }}
+{{- if eq .OCIRepositoryReady.Status "False" }}
+Status:          Last reconciliation failed at {{.OCIRepositoryReady.LastTransitionTime}}
+{{- else }}
+Status:          Last reconciled at {{.OCIRepositoryReady.LastTransitionTime}}
+{{- end }}
+Message:         {{.OCIRepositoryReady.Message}}
+{{- else }}
+Status:          Unknown
+{{- end }}
+{{- end }}
 `
 
 	traceResult := struct {
@@ -522,12 +576,15 @@ Status:        Unknown
 		ObjectNamespace     string
 		HelmRelease         *helmv2.HelmRelease
 		HelmReleaseReady    *metav1.Condition
-		HelmChart           *sourcev1b2.HelmChart
+		HelmChart           *sourcev1.HelmChart
 		HelmChartReady      *metav1.Condition
 		GitRepository       *sourcev1.GitRepository
 		GitRepositoryReady  *metav1.Condition
-		HelmRepository      *sourcev1b2.HelmRepository
+		HelmRepository      *sourcev1.HelmRepository
 		HelmRepositoryReady *metav1.Condition
+		OCIRepository       *sourcev1.OCIRepository
+		OCIRepositoryReady  *metav1.Condition
+		Annotations         map[string]string
 	}{
 		ObjectName:          obj.GetKind() + "/" + obj.GetName(),
 		ObjectNamespace:     obj.GetNamespace(),
@@ -539,6 +596,8 @@ Status:        Unknown
 		GitRepositoryReady:  hrGitRepositoryReady,
 		HelmRepository:      hrHelmRepository,
 		HelmRepositoryReady: hrHelmRepositoryReady,
+		OCIRepository:       hrOCIRepository,
+		OCIRepositoryReady:  hrOCIRepositoryReady,
 	}
 
 	t, err := template.New("tmpl").Parse(traceTmpl)
